﻿local addonName, addonTable = ...
local L = addonTable.L
local PriorityList = LibStub("AceAddon-3.0"):GetAddon("PriorityList")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog   = LibStub("AceConfigDialog-3.0")

local tinsert, tconcat, tremove = table.insert, table.concat, table.remove
local fmt, tostring, string_char = string.format, tostring, string.char
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local loadstring, assert, error = loadstring, assert, error
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget
local bit_band, bit_lshift, bit_rshift = bit.band, bit.lshift, bit.rshift

-- string colors
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local function AddColor(str,color)
 return color..str.."|r"
end

local bytetoB64 = {
    [0]="a","b","c","d","e","f","g","h",
    "i","j","k","l","m","n","o","p",
    "q","r","s","t","u","v","w","x",
    "y","z","A","B","C","D","E","F",
    "G","H","I","J","K","L","M","N",
    "O","P","Q","R","S","T","U","V",
    "W","X","Y","Z","0","1","2","3",
    "4","5","6","7","8","9","(",")"
}

local B64tobyte = {
      a =  0,  b =  1,  c =  2,  d =  3,  e =  4,  f =  5,  g =  6,  h =  7,
      i =  8,  j =  9,  k = 10,  l = 11,  m = 12,  n = 13,  o = 14,  p = 15,
      q = 16,  r = 17,  s = 18,  t = 19,  u = 20,  v = 21,  w = 22,  x = 23,
      y = 24,  z = 25,  A = 26,  B = 27,  C = 28,  D = 29,  E = 30,  F = 31,
      G = 32,  H = 33,  I = 34,  J = 35,  K = 36,  L = 37,  M = 38,  N = 39,
      O = 40,  P = 41,  Q = 42,  R = 43,  S = 44,  T = 45,  U = 46,  V = 47,
      W = 48,  X = 49,  Y = 50,  Z = 51,["0"]=52,["1"]=53,["2"]=54,["3"]=55,
    ["4"]=56,["5"]=57,["6"]=58,["7"]=59,["8"]=60,["9"]=61,["("]=62,[")"]=63
}

--This code is based on the Encode7Bit algorithm from LibCompress
--Credit goes to Galmok of European Stormrage (Horde), galmok@gmail.com
local encodeB64Table = {}

function PriorityList:encodeB64(str)
    local B64 = encodeB64Table;
    local remainder = 0;
    local remainder_length = 0;
    local encoded_size = 0;
    local l=#str
    for i=1,l do
        code = string.byte(str, i);
        remainder = remainder + bit_lshift(code, remainder_length);
        remainder_length = remainder_length + 8;
        while(remainder_length) >= 6 do
            encoded_size = encoded_size + 1;
            B64[encoded_size] = bytetoB64[bit_band(remainder, 63)];
            remainder = bit_rshift(remainder, 6);
            remainder_length = remainder_length - 6;
        end
    end
    if remainder_length > 0 then
        encoded_size = encoded_size + 1;
        B64[encoded_size] = bytetoB64[remainder];
    end
    return table.concat(B64, "", 1, encoded_size)
end

local decodeB64Table = {}

function PriorityList:decodeB64(str)
    local bit8 = decodeB64Table;
    local decoded_size = 0;
    local ch;
    local i = 1;
    local bitfield_len = 0;
    local bitfield = 0;
    local l = #str;
    while true do
        if bitfield_len >= 8 then
            decoded_size = decoded_size + 1;
            bit8[decoded_size] = string_char(bit_band(bitfield, 255));
            bitfield = bit_rshift(bitfield, 8);
            bitfield_len = bitfield_len - 8;
        end
        ch = B64tobyte[str:sub(i, i)];
        bitfield = bitfield + bit_lshift(ch or 0, bitfield_len);
        bitfield_len = bitfield_len + 6;
        if i > l then
            break;
        end
        i = i + 1;
    end
    return table.concat(bit8, "", 1, decoded_size)
end

local Compresser = LibStub:GetLibrary("LibCompress")
local Encoder = Compresser:GetAddonEncodeTable()

function PriorityList:TableToString(inTable, forChat)
    local serialized = PriorityList:Serialize(inTable);
    local compressed = Compresser:Compress(serialized);
    if(forChat) then
        return PriorityList:encodeB64(compressed);
    else
        return Encoder:Encode(compressed);
    end
end

function PriorityList:StringToTable(inString, fromChat)
    local decoded;
    if(fromChat) then
        decoded = PriorityList:decodeB64(inString);
    else
        decoded = Encoder:Decode(inString);
    end
    local decompressed, errorMsg = Compresser:Decompress(decoded);
    if not(decompressed) then
        return "Error decompressing: "..errorMsg;
    end
    local success, deserialized = PriorityList:Deserialize(decompressed);
    if not(success) then
        return "Error deserializing "..deserialized;
    end
    return deserialized;
end

function PriorityList:ImportTimeoutTimer(arg)
 if import_var.Items_List ~= nil and table.getn(import_var.Items_List) > 0 then
  PriorityList:CancelTimer(import_var.import_timer,true)
 else
  if import_var.counter > 19 then --20 seconds timeout
   imp_export.impstatus.name = L["importstatus"]..L["idle"]
   PriorityList:CloseImport_Interface()
   PriorityList:CancelTimer(import_var.import_timer,true)
   AceConfigRegistry:NotifyChange("PriorityList")
   DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["nolistreceived"],smc_r,smc_g,smc_b)
  else --in progress
   imp_export.impstatus.name = L["importstatus"]..L["importprogress"].." "..tostring(30-import_var.counter)
   AceConfigRegistry:NotifyChange("PriorityList")
  end
 end
 import_var.counter = import_var.counter +1
end

PriorityList:RegisterComm("PriorityList", function(prefix, message, distribution, sender)
 local received = PriorityList:StringToTable(PriorityList:decodeB64(message));
 if prefix == "PriorityList" and received and type(received) == "table" then -- receiving items list
  wipe(import_var.Items_List)
  import_var.Items_List = received
  imp_export.importbtn.name = L["importlist"]
  imp_export.importbtn.disabled = true
  imp_export.btn_impall.disabled = false
  imp_export.btn_impend.disabled = false
  imp_export.impstatus.name = L["importstatus"]..string.format(L["listreceivedS"],sender)
  import_var.counter = 0
  PriorityList:PopulateImportList()
  AceConfigRegistry:NotifyChange("PriorityList")
  PriorityList:SendCommMessage("PriorityList", "::msg:received", "WHISPER", sender)

 else -- receiving text message
  newmsg = message
  if prefix == "PriorityList" and strfind(newmsg, "::msg:request") then -- request sent by export btn
   if sender ~= UnitName("player") then -- check for party/raid/officer messages
    i,j = strfind(newmsg, "::msg:request")
    newmsg = string.sub(newmsg,j+1)
    if tonumber(newmsg) < PriorityList.version then
     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>I"].." - "..string.format(L["sendrequest_Verr"],sender,AddColor(sender,GOLD).." "..L["has"]),smc_r,smc_g,smc_b)
     PriorityList:SendCommMessage("PriorityList", "::msg:lowvers", "WHISPER", sender)
    elseif tonumber(newmsg) > PriorityList.version then
     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>I"].." - "..string.format(L["sendrequest_Verr"],sender,AddColor(L["You"],GOLD)..L["have"]),smc_r,smc_g,smc_b)
     PriorityList:SendCommMessage("PriorityList", "::msg:highvers", "WHISPER", sender)
    else
     imp_export.importbtn.name   = L["importlist"].." "..L["from"].." "..sender
     imp_export.importbtn.desc   = string.format(L["importlistdesc"],sender)
     imp_export.importbtn.disabled = false
     import_var.sender = sender
     import_var.ready = true
     AceConfigRegistry:NotifyChange("PriorityList")
     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>I"].." - "..string.format(L["sendrequest"],sender),smc_r,smc_g,smc_b);
    end
   end
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:accept") then
    PriorityList:SendCommMessage("PriorityList", PriorityList:encodeB64(PriorityList:TableToString(Items_List)), "WHISPER", sender)
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["tryingtosend"]..sender.."....",smc_r,smc_g,smc_b);
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:received") then
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..string.format(L["listreceived"],sender),smc_r,smc_g,smc_b);
-- version messages
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:lowvers") then
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["version_low"],smc_r,smc_g,smc_b);
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:highvers") then
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..string.format(L["version_high"],AddColor(sender,GOLD)),smc_r,smc_g,smc_b);
   end
  end
end)
-----------------
-----------------

function PriorityList:Modify_Import_Widgets()
--{"OFFICER","WHISPER","RAID","PARTY"}
local modified = false
 if pl_options["impsndtype"] == 2 then -- WHISPER
  imp_export.exportinput.hidden = false
  imp_export.exportbtn.hidden = true
  modified = true
 elseif pl_options["impsndtype"] > 0 and pl_options["impsndtype"] < 5 then -- OFFICER/RAID/PARTY
  imp_export.exportinput.hidden = true
  imp_export.exportbtn.hidden = false
  imp_export.exportbtn.name = L["exportvia"]..string.lower(L["channel_"..pl_options["impsndtype"]])
  modified = true
 end
 AceConfigRegistry:NotifyChange("PriorityList")
end

function PriorityList:CloseImport_Interface()
 if import_var.Items_List ~= nil then
  wipe(import_var.Items_List)
 end
 table.foreach(waiting_ids,function(k,v) if v["list"] == "import" then v = {} end end)
 import_var.sender = ""
 import_var.ready  = false
 imp_export.btn_impend.disabled = true
 imp_export.btn_impall.disabled = true
 imp_export.importbtn.disabled  = true
 imp_export.impstatus.name  = L["importstatus"]..L["idle"]
 PriorityList.options.args.general.disabled = false
 PriorityList.options.args.search.disabled = false
 wipe(imp_export.itemlist.args)
 imp_export.itemlist.name = ""
 collectgarbage()
end

function PriorityList:UpdateImportListWidgets(itemName,itemLink,itemTexture,index)
 if itemLink ~= nil then
  imp_export.itemlist.args[index].name = itemName
  imp_export.itemlist.args[index].args.descname.name = itemName
  tooltipdesc = PriorityList:ItemTooltipToString(itemLink,itemTexture)
  imp_export.itemlist.args[index].args.item_label.name = tooltipdesc
 else
  tooltipdesc = AddColor(L["IDnotfound"],LIGHT_RED)
  imp_export.itemlist.args[index].name = tooltipdesc
  imp_export.itemlist.args[index].args.descname.name = tooltipdesc
  imp_export.itemlist.args[index].args.item_label.name = tooltipdesc
 end
 AceConfigRegistry:NotifyChange("PriorityList")
end

function PriorityList:PopulateImportList()
------------
 local function Estr_ItemProp(itemID)
 local itemName,itemLink,itemTexture = nil,nil,nil
 itemName, itemLink,_,_,_,_,_,_,_, itemTexture = GetItemInfo(itemID)
  if itemID == 0 then
   itemName = AddColor(L["newitem"],LIGHT_GREEN) 
  elseif itemLink == nil then
   itemName = AddColor(L["IDnotfound"],LIGHT_RED)
   waiting_ids[itemID] = {["list"] = "import"}
  end
  return itemName, itemLink, itemTexture
 end --Estr_ItemProp[erties]
------------
 local function AddItemToList(_index)
 
 local itemName, itemLink, itemTexture = Estr_ItemProp(_index)
 local tooltipdesc = ""
 if itemLink then
  tooltipdesc = PriorityList:ItemTooltipToString(itemLink,itemTexture)
 else
  tooltipdesc = AddColor(L["IDnotfound"],LIGHT_RED)
  itemName = AddColor(L["IDnotfound"],LIGHT_RED)
 end

 -- check for changes
 local impitemdesc = ""
 local needlistdesc = ""
 local found = false
 local changed = false
 --
 import_var.Items_List[_index]["tag"] = "no" -- default (not in the differences list)
 --

  if Items_List[_index] ~= nil then -- item already in list
   if Items_List[_index]["needers"] ~= import_var.Items_List[_index]["needers"] then -- check for changes
    -- button descriptions
    imp_export.itemlist.name = L["importlist"].." da "..import_var.sender
    itemName = AddColor("[MOD]-"..itemName,YELLOW)
    import_var.Items_List[_index]["tag"] = "mod" -- modify (goes to the differences list)
    impitemdesc = format(L["impitemdesc"],L["modify"])
    if Items_List[_index]["needers"] ~= nil then
     needlistdesc = "OLD "..L["needlist"]..": "..AddColor(Items_List[_index]["needers"],LIGHT_RED)
    else
     needlistdesc = "OLD "..L["needlist"]..": "             
    end
    if import_var.Items_List[_index]["needers"] ~= nil then
     needlistdesc = needlistdesc.."\nNEW "..L["needlist"]..": "..AddColor(import_var.Items_List[_index]["needers"],LIGHT_GREEN)
    else
     needlistdesc = needlistdesc.."\nNEW "..L["needlist"]..": "
    end
    changed = true
    --
   end
   found = true
  end
 if not found then
  impitemdesc = format(L["impitemdesc"],L["add"])
  itemName = AddColor("[ADD]-"..itemName,LIGHT_GREEN)
  import_var.Items_List[_index]["tag"] = "add" -- add (goes to the differences list)
  needlistdesc = L["needlist"]..": "..AddColor(import_var.Items_List[_index]["needers"],LIGHT_GREEN)
 end

 if changed or not found then
  import_var.counter = import_var.counter + 1

  imp_export.itemlist.args["item".._index] = {
   name  = itemName,
   type  = "group",
--   order = _index,
   args  = {
           descname =
          {
           order = 1,
           type  = "header",
           name  = "",
          },
         descID   =
          {
           order = 2,
           type  = "description",
           name  = "ID: "..AddColor("#".._index,LIGHT_BLUE),
          },
         descchars=
          {
           order = 3,
           type  = "description",
           name  = needlistdesc,
          },
         h1 =
          {
           order = 4,
           type  = "header",
           name  = "",
          },
   import_item   =
          {
           order = 5,
           type  = "execute",
           name  = L["Execute"],
           desc  = impitemdesc,
           func  = function()
                    if changed then --modify
                     Items_List[_index]["needers"] = import_var.Items_List[_index]["needers"]
                     PriorityList:SysMessage("item modificato: #".._index.." - needers:"..Items_List[_index]["needers"])
                    elseif not found then --add
                     PriorityList:AddItem(_index, import_var.Items_List[_index]["needers"],false,false)
                     PriorityList:SysMessage("item aggiunto: #".._index.." - needers:"..import_var.Items_List[_index]["needers"])
                    else
                     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["error"],smc_r,smc_g,smc_b)
                    end
                    wipe(imp_export.itemlist.args["item".._index].args)
                    imp_export.itemlist.args["item".._index].name = AddColor(L["imported"],LIGHT_BLUE)
                    import_var.Items_List[_index]["tag"] = "no"
                    
                    import_var.counter = import_var.counter -1
                    if import_var.counter == 0 then
                     PriorityList:CloseImport_Interface()
                     imp_export.itemlist.name = AddColor("0 "..L["changes"],LIGHT_RED)
                    else
                     imp_export.itemlist.name = AddColor(import_var.counter.." "..L["changes"],LIGHT_BLUE)
                    end
                   end,
          },
   del_item   =
          {
           order = 6,
           type  = "execute",
           name  = L["DelImp item"],
           desc  = L["delimpdesc"],
           func  = function()
                    wipe(imp_export.itemlist.args["item".._index].args)
                    imp_export.itemlist.args["item".._index].name = AddColor(L["excluded"],RED)
                    import_var.Items_List[_index]["tag"] = "no"
                    
                    import_var.counter = import_var.counter -1
                    if import_var.counter == 0 then
                     PriorityList:CloseImport_Interface()
                     imp_export.itemlist.name = AddColor("0 "..L["changes"],LIGHT_RED)
                    else
                     imp_export.itemlist.name = AddColor(import_var.counter.." "..L["changes"],LIGHT_BLUE)
                    end
                   end,
          },
         h2 =
          {
           order = 10,
           type  = "header",
           name  = "--- Item ---",
          },
 item_label =
          {
           order = 11,
           type  = "description",
           name  = tooltipdesc,
           fontSize = "medium",
          },
   }
  }
  end
 end --[ADD][MOD] items

 function AddMissingItems()

  table.foreach(Items_List,
   function(k,v)
    local found = false
    if import_var.Items_List[k] ~= nil then found = true end

    local itemName, itemLink, itemTexture = Estr_ItemProp(k)
    local tooltipdesc = ""
    if not found then
     if itemLink then
      itemName = AddColor("[DEL]-"..itemName,LIGHT_RED)
      tooltipdesc = PriorityList:ItemTooltipToString(itemLink,itemTexture)
     else 
      tooltipdesc = AddColor(L["IDnotfound"],LIGHT_RED)
      itemName = AddColor("[DEL]-"..L["IDnotfound"],LIGHT_RED)
     end

     import_var.flagged_for_del[k] = true
     import_var.counter = import_var.counter +1     
     imp_export.itemlist.args["item"..k] = {
       name  = itemName,
       type  = "group",
       args  = {
           descname =
          {
           order = 1,
           type  = "header",
           name  = "",
          },
         descID   =
          {
           order = 2,
           type  = "description",
           name  = "ID: "..AddColor("#"..k,LIGHT_BLUE),
          },
         descchars=
          {
           order = 3,
           type  = "description",
           name  = L["needlist"]..": "..AddColor(v["needers"],LIGHT_BLUE),
          },
         h1 =
          {
           order = 4,
           type  = "header",
           name  = "",
          },
   import_item   =
          {
           order = 5,
           type  = "execute",
           name  = L["Execute"],
           desc  = format(L["impitemdesc"],L["delete"]),
           func  = function()
                    if k > 0 then
                     PriorityList:DeleteItem(k)
                    else
                     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["error"],smc_r,smc_g,smc_b)
                    end
                    imp_export.itemlist.args["item"..k].name = AddColor(L["deleted"],LIGHT_BLUE)
                    import_var.flagged_for_del[k] = nil
                    wipe(imp_export.itemlist.args["item"..k].args)
                    
                    import_var.counter = import_var.counter -1
                    if import_var.counter == 0 then
                     PriorityList:CloseImport_Interface()
                     imp_export.itemlist.name = AddColor("0 "..L["changes"],LIGHT_RED)
                    else
                     imp_export.itemlist.name = AddColor(import_var.counter.." "..L["changes"],LIGHT_BLUE)
                    end
                   end,
          },
   del_item   =
          {
           order = 6,
           type  = "execute",
           name  = L["DelImp item"],
           desc  = L["delimpdesc"],
           func  = function()
                    wipe(imp_export.itemlist.args["item"..k].args)
                    imp_export.itemlist.args["item"..k].name = AddColor(L["excluded"],RED)
                    import_var.flagged_for_del[k] = nil
                    
                    import_var.counter = import_var.counter -1
                    if import_var.counter == 0 then
                     PriorityList:CloseImport_Interface()
                     imp_export.itemlist.name = AddColor("0 "..L["changes"],LIGHT_RED)
                    else
                     imp_export.itemlist.name = AddColor(import_var.counter.." "..L["changes"],LIGHT_BLUE)
                    end
                   end,
          },
         h2 =
          {
           order = 10,
           type  = "header",
           name  = "--- Item ---",
          },
 item_label =
          {
           order = 11,
           type  = "description",
           name  = tooltipdesc,
           fontSize = "medium",
          },
   }
  }
    end
   end) --foreach
end--[DEL] items

 wipe(imp_export.itemlist.args)
 PriorityList:SysMessage("Populate items import list")
 table.foreach(waiting_ids,function(k,v) if v["list"] == "import" then v = {} end end)
 if import_var.Items_List ~= nil then
  wipe(import_var.flagged_for_del)
  table.foreach(import_var.Items_List, AddItemToList)
  AddMissingItems()
  if import_var.counter == 0 then
   PriorityList:CloseImport_Interface()
   imp_export.itemlist.name = AddColor("0 "..L["changes"],LIGHT_RED)
  else
   imp_export.itemlist.name = AddColor(import_var.counter.." "..L["changes"],LIGHT_BLUE)
  end
 end
 PriorityList:CancelTimer(import_var.import_timer,true)
 collectgarbage()
end