﻿local addonName, addonTable = ...
local L = addonTable.L
local PriorityList = LibStub("AceAddon-3.0"):GetAddon("PriorityList")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog   = LibStub("AceConfigDialog-3.0")

-- string colors
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local function AddColor(str,color)
 return color..str.."|r"
end

function PriorityList:ItemTooltipToString(itemLink, itemTexture)
 local f = CreateFrame('GameTooltip', 'MyTooltip', UIParent, 'GameTooltipTemplate')
 local desc = ""
 local line = ""

 f:SetOwner(UIParent, 'ANCHOR_NONE')
 f:SetHyperlink(itemLink)
 desc = "|T" ..itemTexture..":32|t "
 local itemlevel_passed = false
 for tti = 1,f:NumLines() do
  local line = _G["MyTooltipTextLeft"..tti]:GetText()
  if tti == 1 then
   desc = AddColor(desc..line,GOLD)
  elseif not itemlevel_passed then
   if string.find(line,L["Item Level "]) then itemlevel_passed = true end
   if string.find(line,L[" Socket"]) or
      string.find(line,L[" Armor"]) then
   desc = desc.."\n"..AddColor(line,GREY)
   else
    desc = desc.."\n"..line
   end
  else
   desc = desc.."\n"..AddColor(line,LIGHT_GREEN)
  end
 end
 f:Hide()
 return desc
end

function PriorityList:PopulateOptions()

 if not PriorityList.options then
  PriorityList.options = 
   {
    order = 1,
    type  = "group",
    name  = L["<addon name>"],
    childGroups = "tab",
	args  = 
    {
     general = 
      {
       order = 1,
       type = "group",
       name = L["itemlist"],
       args = 
        {
         additem =
          {
           order = 11,
           type  = "execute",
           name  = L["Add item"],
           desc  = L["additemdesc"],
           func  = function(info)
                    PriorityList:AddItem(0,L["newneeder"],true,true)
                   end,
          },        
         itemlist = 
          {
           order = 20,
           type  = "group",
           name  = L["itemlist"],
           desc  = "",
           args  = {},
          },
       clearlist  =
          {
           order = 41,
           type  = "execute",
           name  = L["clearlist"],
           desc  = L["clearlistdesc"],
          confirm= true,
           func  = function()
                    PriorityList:ClearList()
                    PriorityList:PopulateItemsList()
                   end,
          },
        }
      }, -- general tab
--

-- OPTIONS
    options = {
           order = 20,
           type = "group",
           name = L["opt_options"],
           args = {
     confirm_edit=
          {
           order = 1,
           type  = "toggle",
           name  = L["confirm_edit"],
           desc  = L["confirm_editdesc"],
           get   = function() return pl_options["confirm_edit"] end,
           set   = function()
                    pl_options["confirm_edit"] = not pl_options["confirm_edit"] 
                    PriorityList:PopulateItemsList() --recreate every widget with the new confirm option
                   end,
          },
     link_addbtn =
          {
           order = 2,
           type  = "toggle",
           name  = L["link_tltaddbtn"],
           desc  = L["link_tltadddesc"],
           get   = function() return pl_options["ttlinkbtn"] end,
           set   = function() pl_options["ttlinkbtn"] = not pl_options["ttlinkbtn"] end,
          },
        }
      }, -- options tab
-- IMPORT/EXPORT
     imp_exp = 
      {
       order = 10,
       type = "group",
       name = L["opt_Import/Export"],
       args = {
        hexport = {
             order = 4,
             type  = "header",
             name  = L["exportbtn"],
         },
         exporttypelist = 
          {
           order = 5,
           type  = "select",
           name  = "",
           desc  = "",
           values= {"Officer","Whisper","Raid","Party"},
           get = function(info)
                  return pl_options["impsndtype"]
                 end,
           set   = function(info, value)
                    pl_options["impsndtype"] = value
                    PriorityList:Modify_Import_Widgets()
                   end,
          },
     exportinput =
          {
           order = 6,
           type  = "input",
           name  = L["exportlist"],
           desc  = L["exportlistdesc"],
           set   = function(info, v) --"WHISPER"
                    if string.lower(v) ~= string.lower(UnitName("player")) then
                     if pl_options["impsndtype"] == 2 then
                      PriorityList:SendCommMessage("PriorityList", "::msg:request"..PriorityList.version, imp_export.exporttypelist.values[pl_options["impsndtype"]], v)
                      DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..string.format(L["requestsent"],v),smc_r,smc_g,smc_b)
                     end
                    else
                     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - non puoi inviare la lista a te stesso, burlone!",smc_r,smc_g,smc_b)
                    end
                   end,
          },
     exportbtn   =
          {
           order = 6,
           type  = "execute",
           name  = L["exportbtn"],
           desc  = L["export___d"],
           func  = function() -- non-whisper
                    if pl_options["impsndtype"] ~= 2 then
                     PriorityList:SendCommMessage("PriorityList", "::msg:request"..PriorityList.version, imp_export.exporttypelist.values[pl_options["impsndtype"]], "")
                     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..string.format(L["requestsent"],L["channel_"..pl_options["impsndtype"]]),smc_r,smc_g,smc_b)
                    else
                     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - Impossibile usare questo pulsante per whisper",smc_r,smc_g,smc_b)
                    end
                   end,
          },
himport = {
             order = 10,
             type  = "header",
             name  = L["importlist"],
         },
       importbtn  =
          {
           order = 12,
           type  = "execute",
           name  = L["importlist"],
           desc  = L["importlistdesc"],
           disabled= true,
           func  = function()
                    if import_var.ready and import_var.sender then
                     PriorityList:SendCommMessage("PriorityList", "::msg:accept", "WHISPER", import_var.sender)
                     imp_export.impstatus.name = L["importstatus"]
                     import_var.counter = 0
                     PriorityList.options.args.general.disabled = true -- do not modify the list while importing/modifying
                     PriorityList.options.args.search.disabled = true
                     import_var.import_timer = PriorityList:ScheduleRepeatingTimer("ImportTimeoutTimer", 1, "importbtn")
                    else
                     import_var.sender = ""
                     import_var.ready = false
                     imp_export.importbtn.name = L["importlist"]
                     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["importerror"],smc_r,smc_g,smc_b);
                    end
                    imp_export.importbtn.disabled = true
                    GameTooltip:Hide()
                    AceConfigRegistry:NotifyChange("PriorityList")
                   end,
          },
  impstatus    = 
          {
           order = 13,
           type  = "description",
           name  = L["importprogress"],
        fontSize = "medium",
          },
     btn_impall   =
          {
           order    = 14,
           type     = "execute",
           name     = L["impall"],
           disabled = true,
           func     = function()
                       table.foreach(import_var.Items_List,
                        function(k,v)
                         if v["tag"] == "add" then
                          PriorityList:AddItem(k,v["needers"],false,false)
                         elseif v["tag"] == "mod" then
                          Items_List[k]["needers"] = v["needers"]
                         end
                        end)
                       table.foreach(import_var.flagged_for_del, function(k,v)
                        PriorityList:DeleteItem(k,false)
                       end)
                       PriorityList:CloseImport_Interface()
                       PriorityList:PopulateItemsList()
                      end,
          },
     btn_impend   =
          {
           order = 15,
           type  = "execute",
           name  = L["end"],
           disabled= true,
           func  = function()
                    PriorityList:CloseImport_Interface()
                    PriorityList:PopulateItemsList()
                   end,
          },
hconfirm = {
             order = 20,
             type  = "header",
             name  = "",
         },
       itemlist  =
          {
           order  = 12,
           type   = "group",
           name   = "",
           args   = { }
          },
         }
      }, -- import/export tab
-- SEARCH
    search  = {
           order = 20,
           type = "group",
           name = L["opt_search"],
           args = {
s_searchlist    = {
                 order = 40,
                 type = "group",
                 name = L["s_search"],
                 args = { }
                  },
     searchbtn    = {
           order  = 11,
           type   = "execute",
           name   = L["s_search"],
         func   = function ()
                   local found,tmp_names
                   wipe(opt_search.s_searchlist.args)
                   if Items_List ~= nil then
                    table.foreach(boss_drops_table,
                     function(k,v)
                      -- boss sub section
                      found = false
                      for diff_index,boss_difficulty in pairs(v) do
                      for ID_index,boss_itemID in pairs (boss_difficulty) do
                       if not opt_search.s_searchlist.args[diff_index] then
                       -- difficulty sub section if it doesn't exist
                         opt_search.s_searchlist.args[diff_index] = {name = AddColor(diff_index,GOLD), type = "group", args = {}}
                       end
                       if not opt_search.s_searchlist.args[diff_index].args[k] then
                       -- difficulty sub section if it doesn't exist
                         opt_search.s_searchlist.args[diff_index].args[k] = {name = AddColor(k,LIGHT_BLUE), type = "group", args = {}}
                       end

                       if Items_List[boss_itemID] ~= nil then
                        tmp_names = Items_List[boss_itemID]["needers"]
                        names_table = {}
                        for word in string.lower(tmp_names):gmatch("%a+") do table.insert(names_table, word) end
                        if names_table then
                         for a,b in pairs(names_table) do
                          if not opt_search.s_searchlist.args[diff_index].args[k].args[b] then
                       -- character sub section if it doesn't exist
                           opt_search.s_searchlist.args[diff_index].args[k].args[b] = {name = b, type = "group", args = {}}
                          end
                       -- needed item description       
                         local itemName, itemLink,_,_,_,_,_,_,_, itemTexture = GetItemInfo(boss_itemID)
                         local tmp_getname = ""
                         if itemLink then
                          tmp_getname = PriorityList:ItemTooltipToString(itemLink,itemTexture)
                          if a == 1 then
                           tmp_getname = tmp_getname.."\n"..AddColor(" #"..a.." ("..Items_List[boss_itemID]["needers"]..")",LIGHT_RED)
                          elseif a > 1 and a < 4 then
                           tmp_getname = tmp_getname.."\n"..AddColor(" #"..a.." ("..Items_List[boss_itemID]["needers"]..")",YELLOW)
                          elseif a > 3 then
                           tmp_getname = tmp_getname.."\n"..AddColor(" #"..a.." ("..Items_List[boss_itemID]["needers"]..")",GREEN)
                          end
                         else
                          tmp_getname = AddColor(L["IDnotfound"],LIGHT_RED)
                         end
                          tmp_getname = tmp_getname.."\n---\n---\n"
                          opt_search.s_searchlist.args[diff_index].args[k].args[b].args[boss_itemID] = {type = "description", fontSize = "medium", name = tmp_getname}
                        end
                      end --item ids
                      end --difficulty
                     end
                     end
                   end) -- table.foreach
                  end
                 end,
           },
         },
      }, --search tab
    }
   }
-- abbreviations
 -- general
 opt_general = PriorityList.options.args.general.args
 -- import/export
 imp_export  = PriorityList.options.args.imp_exp.args
 -- options (useless atm)
-- opt_options = PriorityList.options.args.options.args
 -- search
 opt_search  = PriorityList.options.args.search.args

  PriorityList:PopulateItemsList()
 end
end